************************************************
*
* Program	:	Tagmain.prg
*
* By		:	Michael Meer
*				CIS # 72426,3467
*
* Created on:	Jan 1, 1995
* Last Edit	:	
*
* Purpose	:	This is the main program for the Tagmnt Exe. Based upon the parameter used 
*				when it is called, the program will create dbfs with dbf & tag info or  
*				delete tags on one or more dbfs and recreate them. If no parameter is passed, 
*				the program will return without doing anything. 
*
*				NOTE: The program will ignore any databases listed in the file IGNORE.TXT. 
*				I've included the program dbfs and the Foxuser.dbf as examples.
*
*				NOTE: No prompting internally is used so that this program can be called by 
*				overnight routines for unattend operation. If you use this program in your
*				programs, you may want to use an "Are you sure" window before executing the
*				tagmnt.exe function.
*	
* Parameters:	pcMethod	<Required for any processes to occur>
*					Possible Values:
*						"RELOAD" - Reloads the dbf field names and tag names
*						"REINDEX" - Deletes tags and recreates them	
*				pcDbfName	<Optional>
*					Possible Values:
*						<dbf name> - Will only reload or reindex a single dbf
*				pcLogit		<Optional>
*					Possible Values:
*						.T. - Will log results to TAGLOG.DBF
*				pcPackit	<Optional>
*					Possible Values:
*						.T. - Will pack databases at time of rebuilding 
*						
*
*						
* Examples	:	DO Tagmnt WITH "RELOAD" - Will reload entire dbf into tables
*				DO Tagmnt WITH "REINDEX" - Will delete all tags and recreate them for all dbfs
*
* Files Used:	Dbflist.*	-	Holds field information for each dbf
*				Taglist.*	- 	Holds tag information for each dbf
*				Taglog.*	-	Holds results of operations if flag is includeded in the parameters
*				Tagmnt.exe	-	Main executable
*
***************************************************				
PARAMETER pcMethod, pcDbfName, pcLogit, pcPackit
IF PARAMETERS() = 0
	** Early Exit due to lack of parameters **
	RETURN
ENDIF
IF PARAMETERS() < 3
	pcLogit = .F.
	pcPackit = .F.
ENDIF
IF PARAMETERS() < 2
	pcDbfName = ''
ENDIF

** Capitialize all parameters **
pcMethod = UPPER( pcMethod)
pcDbfName = UPPER( pcDbfName)

** Save environment **
lcEnvSafety		= SET("SAFETY")
lcEnvTalk		= SET("TALK")
lcEnvEcho		= SET("ECHO")
lcEnvCurrentDBF	= ALIAS()

SET SAFETY OFF
SET TALK OFF
SET ECHO OFF

** Set error handling routine **
ON ERROR DO ERRHANDLER WITH ERROR(), MESSAGE()

** Save Open Files **
DIMENSION laOpenDbfs[ 225,2]
DO DBFS WITH "SAVE"

** Close Databases **
CLOSE DATABASES

** Open Dbflist & TagList **
USE Dbflist IN 1 EXCLUSIVE
USE Taglist IN 2 EXCLUSIVE
USE Taglog IN 3 SHARED

** Read in Ignored Dbf List **
DIMENSION laIgnoreList[1]
DO GETIGNORE

** Initialize Any Global Vars **
glError = .F.


** Call subroutines based upon parameters **
IF NOT glError
	DO CASE
		CASE pcMethod = "RELOAD"
			DO DBFRELOAD

		CASE pcMethod = "REINDEX"
			DO REBUILD
	ENDCASE
ENDIF

** Close all Databases, Again! **
CLOSE DATABASE

** Reset Environment **
IF lcEnvSafety = "ON"
	SET SAFETY ON
ENDIF
IF lcEnvTalk = "ON"
	SET TALK ON
ENDIF
IF lcEnvEcho = "ON"
	SET ECHO ON
ENDIF

** Reopen Dbfs **
DO DBFS WITH "OPEN"

** Clear any wait windows **
WAIT CLEAR 

** Done **
RETURN


**  **  **  **  **  **  **  **  
  **  **  **  **  **  **  **
**  **  **  **  **  **  **  **
************************************************
*
* Function	:	DBFS
* By		:	Michael Meer
* Created on:	Jan 1, 1995
* Last Edit	:	
*
* Purpose	:	Saves the currently opened dbfs names, paths and order settings to an array
*				so that the program can reset the environment when completed.
*	
* Parameters:	pcSaveMethod 	<Required>
*					Values:	SAVE - Loads array
*							OPEN - Opens dbfs based upon info in array
*							
*
************************************************
FUNCTION DBFS
PARAMETER pcSaveMethod
ON ERROR DO ERRHANDLER WITH ERROR(), MESSAGE(), laOpenDbfs[ jnLoopCount, 1]
FOR jnLoopCount = 1 TO 225
	SELECT (jnLoopCount)
	DO CASE 
		CASE pcSaveMethod = "SAVE"
			laOpenDbfs[ jnLoopCount, 1] = ALLTRIM( DBF())
			laOpenDbfs[ jnLoopCount, 2] = ALLTRIM( ORDER())
			
		CASE pcSaveMethod = "OPEN"
			USE (laOpenDbfs[ jnLoopCount, 1]) ORDER (laOpenDbfs[ jnLoopCount, 2])
	ENDCASE
ENDFOR
ON ERROR DO ERRHANDLER WITH ERROR(), MESSAGE()
RETURN
**  **  **  **  **  **  **  **  
**  **  **  **  **  **  **  **  


************************************************
*
* Function	:	GETIGNORE
* By		:	Michael Meer
* Created on:	Jan 1, 1995
* Last Edit	:	
*
* Purpose	:	Loads database names into a string for later processing in the program.
*	
* Parameters:	None
*
************************************************
FUNCTION GETIGNORE
jnFileHandle = FOPEN( 'IGNORE.TXT')
jnCounter = 1
IF jnFileHandle > -1
	DO WHILE NOT FEOF( jnFileHandle)
		jcLineText = FGETS( jnFileHandle)
		IF LEFT( jcLineText, 1) != "*" AND NOT EMPTY( jcLineText)
			IF jnCounter > 1
				DIMENSION laIgnoreList[ jnCounter]
			ENDIF
			laIgnoreList[ jnCounter] = ALLTRIM( jcLineText)
			jnCounter = jnCounter + 1
		ENDIF
	ENDDO
	= FCLOSE( jnFileHandle)
ENDIF
RETURN
**  **  **  **  **  **  **  **  			
**  **  **  **  **  **  **  **  


************************************************
*
* Function	:	LOGACTION
* By		:	Michael Meer
* Created on:	Jan 1, 1995
* Last Edit	:	
*
* Purpose	:	Creates an entry in Taglog.dbf when the correct parameters are passed to the 
*				Tagmnt.prg program. 
*	
* Parameters:	pcProgName <Required> 		Program Name or Function Name
*				pcProgMessage <Required>	Message to be logged 
*							
*
************************************************
FUNCTION LOGACTION
PARAMETER pcProgName, pcProgMessage
DIMENSION jaTagLogInfo[4]
IF PARAMETERS() = 2
	jaTagLogInfo[1] = pcProgName
	jaTagLogInfo[2] = pcProgMessage
	jaTagLogInfo[3] = TIME()
	jaTagLogInfo[4] = DATE()
	INSERT INTO Taglog FROM ARRAY jaTagLogInfo
ENDIF
RETURN
**  **  **  **  **  **  **  **  
**  **  **  **  **  **  **  **  


************************************************
*
* Function	:	DBFRELOAD
* By		:	Michael Meer
* Created on:	Jan 1, 1995
* Last Edit	:	
*
* Purpose	:	Clears Dbflist.dbf and then reloads the database name and field information. 
*				Also, while the target database(s) are open, the program will get all the 
*				tag information from the database and place it in taglist.dbf.
*	
* Parameters:	None
*
************************************************
FUNCTION DBFRELOAD
** Load Dbf names into an array **
IF EMPTY( pcDbfName)
	jnDbfCount = ADIR( jaDbfList, "*.DBF")
ELSE
	jnDbfCount = ADIR( jaDbfList, pcDbfName)
ENDIF


** If there are dbfs to record... **
IF jnDbfCount > 0
	** Record action to Log dbf if requested **
	IF pcLogit
		DO LOGACTION WITH "DBFRELOAD", "Starting Reload..."
	ENDIF

	** Clean out Taglist.dbf **
	SELECT Taglist
	IF EMPTY( pcDbfName)
		ZAP
	ELSE
		DELETE ALL FOR dbf_name = pcDbfName
		PACK
	ENDIF
	
	** Setup arrays for later use **	
	DIMENSION jaTagData[ 8], jaCleanArray[ 8]
	SCATTER TO ja_tagdata BLANK
	SCATTER TO ja_cleanarray BLANK
	
	** Clean out Dbflist.dbf **
	SELECT Dbflist
	IF EMPTY( pcDbfName)
		ZAP
	ELSE
		DELETE ALL FOR dbf_name = pcDbfName
		PACK
	ENDIF
	
	** Sort into alphabetical order **
	= ASORT( jaDbfList, 1)
	
	** Loop & load info **
	FOR jnLoopCount = 1 TO jnDbfCount
		** Check to see if the dbf is in the ignore list **
		jlProcessThisOne = .T.
		FOR jnCheckLoop = 1 TO ALEN( laIgnoreList, 1)
			IF jaDbfList[ jnLoopCount, 1] =  laIgnoreList[ jnCheckLoop]
				jlProcessThisOne = .F.
				EXIT
			ENDIF
		ENDFOR
		IF jlProcessThisOne
			** Record action to Log dbf if requested **
			IF pcLogit
				DO LOGACTION WITH "DBFRELOAD", "     Loading " + PROPER( jaDbfList[ jnLoopCount, 1])
			ENDIF

			** If not, display name to show user that dbf is being processed **
			WAIT WINDOW NOWAIT PADR( jaDbfList[ jnLoopCount, 1], 60)
			
			** Select next open work area **
			SELECT 0
			
			** Use dbf **
			USE jaDbfList[ jnLoopCount, 1] SHARED
			
			** Create Dbftemp.dbf with structure info in it **
			COPY STRUCTURE EXTENDED TO Dbftemp
			SELECT Dbflist
			APPEND FROM Dbftemp
			REPLACE ALL dbf_name WITH jaDbfList[ jnLoopCount, 1] FOR EMPTY( dbf_name)
			
			** Make sure correct dbf is selected **
			SELECT (jaDbfList[ jnLoopCount, 1])
			
			** Load Tag information **
			jnTagno = 1
			DO WHILE NOT EMPTY( TAG( ALIAS(), jnTagno))
				** Display Tag info to user **
				WAIT WINDOW NOWAIT PADR( jaDbfList[ jnLoopCount, 1] + " Tag: " + TAG( ALIAS(), jnTagno), 60)
				jaTagData[ 1] = jaDbfList[ jnLoopCount, 1] 
				jaTagData[ 2] = TAG( ALIAS(), jnTagno)
				jaTagData[ 3] = KEY( ALIAS(), jnTagno)
				jaTagData[ 4] = FOR( ALIAS(), jnTagno)
				jaTagData[ 5] = UNIQUE( ALIAS(), jnTagno)
				jaTagData[ 6] = DESCENDING( ALIAS(), jnTagno)
				jaTagData[ 7] = DATE()
				jaTagData[ 8] = jnTagno
				INSERT INTO Taglist FROM ARRAY jaTagData
				= ACOPY( jaCleanArray, jaTagData)
				jnTagno = jnTagno + 1
			ENDDO

			** Close database **
			USE IN ( jaDbfList[ jnLoopCount, 1])
		ENDIF
	ENDFOR		
	** Record action to Log dbf if requested **
	IF pcLogit
		DO LOGACTION WITH "DBFRELOAD", "Finished Loading!"
	ENDIF
	
	** Delete Dbftemp file **
	DELETE FILE "DBFTEMP.DBF"
	
	** Clear last wait window **
	WAIT CLEAR
ENDIF
**  **  **  **  **  **  **  **  			
**  **  **  **  **  **  **  **  


************************************************
*
* Function	:	REBUILD
* By		:	Michael Meer
* Created on:	Jan 1, 1995
* Last Edit	:	
*
* Purpose	:	This function will delete all tags from a database and then recreates the 
*				tags from the information stored in Taglist.dbf.
*
* Parameters:	None
*
************************************************
FUNCTION REBUILD

** Record action to Log dbf if requested **
IF pcLogit
	DO LOGACTION WITH "REBUILD", "Starting Rebuild Process"
ENDIF

* Check to see if a single database name was included in the parameter statement *
IF EMPTY( pcDbfName)
	* Use SQL to load the array from the dbf Dbflist.dbf *
	SELECT dbf_name FROM Dbflist GROUP BY dbf_name INTO ARRAY jaDbfList
	jnDbfCount = _tally
	* Sort it *
	= ASORT( jaDbfList, 1)
ELSE
	* Create one cell array *
	DIMENSION jaDbfList[1]
	jaDbfList[1] = pcDbfName
	jnDbfCount = 1
ENDIF

** Now we can have a generic routine to rebuild the tags... **

FOR jnLoopCount = 1 TO jnDbfCount 
	** Record action to Log dbf if requested **
	IF pcLogit
		DO LOGACTION WITH "REBUILD", "Retagging " + PROPER( jaDbfList[ jnLoopCount, 1])
	ENDIF
 
	** Set error handler **
	ON ERROR DO ERRHANDLER WITH ERROR(), MESSAGE(), jaDbfList[ jnLoopCount, 1]
	glError = .F.
	
	** Select Open Workarea & Use Dbf *
	SELECT 0
	USE (jaDbfList[ jnLoopCount, 1]) EXCLUSIVE
	
	IF NOT glError
		** Load tag info into array via SQL **
		SELECT * FROM Taglist INTO ARRAY jaTagList ORDER BY tag_number WHERE dbf_name = jaDbfList[ jnLoopCount, 1]
		jnTagCount = _tally
		
		** If records found... Process them... **
		IF jnTagCount > 0
			** But first, delete all tags from database **
			DELETE TAG ALL
			
			** Display name to show user that dbf is being processed **
			WAIT WINDOW NOWAIT PADR( jaDbfList[ jnLoopCount, 1], 60)

			
			** If pack is requested from parameters, pack here **
			IF pcPackIt
				IF pcLogit
					DO LOGACTION WITH "REBUILD", "     ** Packing **"
				ENDIF
				** Display more info... **
				WAIT WINDOW NOWAIT PADR( jaDbfList[ jnLoopCount, 1] + " Packing Now...", 60) 
				PACK
			ENDIF
			
			FOR jnLoop2Count = 1 TO jnTagCount
				IF pcLogit
					DO LOGACTION WITH "REBUILD", "     Creating Tag: " + LOWER( ALLTRIM( jaTagList[ jnLoop2Count, 2])) 
				ENDIF
				** Display more info... **
				WAIT WINDOW NOWAIT PADR( jaDbfList[ jnLoopCount, 1] + " Tag: " + ALLTRIM( jaTagList[ jnLoop2Count, 2]), 60)
				** Create a command line with all instructions for creating the tag **
				jcCommandLine = "INDEX ON " + ALLTRIM( jaTagList[ jnLoop2Count, 3])  + ;
					" TAG " + ALLTRIM( jaTagList[ jnLoop2Count, 2]) + ;
					IIF( EMPTY( jaTagList[ jnLoop2Count, 4]), "", ;
						" FOR " + ALLTRIM( jaTagList[ jnLoop2Count, 4])) + ;
					IIF( jaTagList[ jnLoop2Count, 5], " UNIQUE ", "") + ;
					IIF( jaTagList[ jnLoop2Count, 6], " DESCENDING ", "") 
					
				** Execute Command Line via Macro command **
				&jcCommandLine
			ENDFOR
	
			** Now that it is done, close database **
			USE IN ( jaDbfList[ jnLoopCount, 1]) 
		ENDIF
	ENDIF
ENDFOR


************************************************
*
* Function	:	ERRHANDLER
* By		:	Michael Meer
* Created on:	Jan 1, 1995
* Last Edit	:	
*
* Purpose	:	This Routine Does basic error handling...
*	
* Parameters:	pcErrorNum	<Required>
*					** Holds error # **
*				pcErrorMsg	<Required>
*					** Holds error text **
*				pcDbfName	<Optional>
*					** Optional - holds dbf name that was trying to be accessed **
*
************************************************
FUNCTION ERRHANDLER
PARAMETER pcErrorNum, pcErrorMsg, pcDbfName
IF USED( 'Taglog')
	IF pcErrorNum = 1705
		DO LOGACTION WITH pcMethod, "Error: Unable to get exclusive access to " + pcDbfName
	ELSE
		DO LOGACTION WITH pcMethod, "Error: " + ALLTRIM( STR( pcErrorNum)) + " " + pcErrorMsg
	ENDIF
ENDIF
glError = .T.
RETURN
